import java.io.*;
import java.util.*;

import org.jdom2.*;



/**
 * Takes the specified template for a Feature IDE model file and adds all available
 * sensorconfig/ gazebo model files.
 */
public class ModelSetup extends XMLHelper{

	/**
	 * Obtains an attribute value of the input/ output entries specified in modelsetup.config.
	 * 
	 * @param doc		The JDOM Document (modelsetup.config)
	 * @param entry		String formed as "{input or output}/{entity}"
	 * @param val		Attribute name as string
	 * @return path as specified in modelsetup.config
	 */
	private static String getConfigVal(Document doc, String str, String val) {
		List<Attribute> valList = xQueryAttributes(doc, "//"+str+"/@"+val);
		if(valList.size()!=1) {
			System.err.println("Error while reading input/ output "+val+" value from modelsetup.config.");
			System.exit(-1);
		}

		return valList.get(0).getValue();
	}
	
	/**
	 * @param args
	 */
	public ModelSetup {
		System.out.println("ModelSetup started.");
		
		try {
			//######### Input ###############################			
			Document ModelSetupConfig = readXml("./modelsetup.config");
			
			String in_mPath = getConfigVal(ModelSetupConfig, "input/model", "path");
			String in_scPath = getConfigVal(ModelSetupConfig, "input/sensorconfig", "path");
			String in_gmPath = getConfigVal(ModelSetupConfig, "input/gazebomodel", "path");
			String out_mPath = getConfigVal(ModelSetupConfig, "output/model", "path");
			String in_mName = getConfigVal(ModelSetupConfig, "input/model", "name");
			String out_mName = getConfigVal(ModelSetupConfig, "output/model", "name");
			
			Document model = readXml(in_mPath+in_mName);

			List<String> scList = getFileNames(in_scPath);
			List<String> gmList = getFileNames(in_gmPath);
			
			String scEnding = ".xml";
			String gmEnding = ".gazebo.xacro";
			
			//Check whether feature input was empty
			if(scList.isEmpty()) {
				System.err.println("Error: No sensorconfig files found in \'"+in_scPath+"\'.");
				System.exit(-1);
			}
			if(gmList.isEmpty()) {
				System.err.println("Error: No gazebo model files found in \'"+in_gmPath+"\'.");
				System.exit(-1);
			}
			
			//Check for well formed file names
			for(String sc : scList) {
				if(!sc.endsWith(scEnding)) {
					System.err.println("Error: sensorconfig file \'"+sc+"\' does not have an \'"+scEnding+"\' ending.");
					System.exit(-1);
				}
			}
			for(String gm : gmList) {
				if(!gm.endsWith(gmEnding)) {
					System.err.println("Error: gazebo model file \'"+gm+"\' does not have an \'"+gmEnding+"\' ending.");
					System.exit(-1);
				}
			}
			//###############################################
			
			
			//######### Operate on model ###################
			//Get Elements where features are to be added
			Element templates = null;
			List<Element> templatesList = xQueryElements(model, "//*[@name='Templates']");
			if(templatesList.size()==1) {
				templates = templatesList.get(0);
			}
			else {
				System.err.println("Error when parsing for \'Templates\' in \'"+in_mName+"\'.");
				System.exit(-1);
			}
			
			Element custom = null;
			List<Element> customList = xQueryElements(model, "//*[@name='Custom']");
			if(customList.size()==1) {
				custom = customList.get(0);
			}
			else {
				System.err.println("Error when parsing for \'Custom\' in \'"+in_mName+"\'.");
				System.exit(-1);
			}
			
			//Add features
			for(String sc : scList) {
				String featureName = sc.substring(0, sc.length() - scEnding.length());
				Element feature = new Element("feature").setAttribute("name", featureName);
				custom.addContent(feature);
			}
			
			for(String gm : gmList) {
				String featureName = gm.substring(0, gm.length() - gmEnding.length());
				Element feature = new Element("feature").setAttribute("name", featureName);
				templates.addContent(feature);
			}
			//###############################################
			
			
			//######### Output result #######################
			saveAsXml(model, out_mPath+out_mName);
			//###############################################
		}
		catch (JDOMException | IOException e) {
			e.printStackTrace();
		}

		System.out.println("ModelSetup finished.");
	}
}



